@@ -102,6 +102,13 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "myselect.h"
 #endif
 
+/* Starting with Tcl 8.4, many APIs offer const-correctness.
+   Unfortunately, making _tkinter correct for this API means to break
+   earlier versions. USE_COMPAT_CONST allows to make Pax work with both
+   8.4 and earlier versions. Once Tcl releases before 8.4 don't need to
+   be supported anymore, this should go. */
+#define USE_COMPAT_CONST
+
 #include <tcl.h>
 #include <tk.h>
 
@@ -799,7 +806,8 @@ Tkapp_AddErrorInfo(PyObject *self, PyObject *args)
 static PyObject *
 SetVar(PyObject *self, PyObject *args, int flags)
 {
-	char *name1, *name2, *ok, *s;
+	char *name1, *name2, *s;
+	const char *ok;
 	PyObject *newValue;
 	PyObject *tmp;
 
@@ -854,7 +862,8 @@ Tkapp_GlobalSetVar(PyObject *self, PyObject *args)
 static PyObject *
 GetVar(PyObject *self, PyObject *args, int flags)
 {
-	char *name1, *name2=NULL, *s;
+	char *name1, *name2=NULL;
+	const char *s;
 	PyObject *res = NULL;
 
 	if (!PyArg_ParseTuple(args, "s|s", &name1, &name2))
@@ -1714,6 +1723,38 @@ Tkapp_UTF8_to_Latin1 (PyObject *self, PyObject *args)
 #endif
 
 
+static PyObject *
+Tkapp_UTF8_to_System (PyObject *self, PyObject *args)
+#if TKMAJORMINOR < 8001
+{
+    PyObject * string;
+
+    if (!PyArg_ParseTuple(args, "S", &string))
+	return NULL;
+
+    Py_INCREF(string);
+    return string;
+}
+#else
+{
+    PyObject * utf8;
+    PyObject * system;
+    Tcl_DString dstring;
+
+    if (!PyArg_ParseTuple(args, "S", &utf8))
+	return NULL;
+
+    Tcl_UtfToExternalDString(NULL, PyString_AsString(utf8),
+			     PyString_Size(utf8), &dstring);
+
+    system = PyString_FromString(dstring.string);
+
+    Tcl_DStringFree(&dstring);
+
+    return system;
+}
+#endif
+
 
 
 /**** Tkapp Method List ****/
@@ -1755,6 +1796,7 @@ static PyMethodDef Tkapp_methods[] =
 	{"quit", 	       Tkapp_Quit, 1},
 	{"interpaddr",         Tkapp_InterpAddr, 1},
 	{"utf8_to_latin1",     Tkapp_UTF8_to_Latin1, 1},
+	{"utf8_to_system",     Tkapp_UTF8_to_System, 1},
 	{NULL, 		       NULL}
 };
 
