@@ -178,7 +178,7 @@ static void
 region_dealloc(PaxRegionObject  *self)
 {
     XDestroyRegion(self->region);
-    free(self);
+    PyMem_DEL(self);
 }
 
 
