@@ -1,20 +1,20 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003 by Bernhard Herzog
 # Some code for Text elements contributed by Paul Giotta (2002)
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or (at
-# your option) any later version.
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Library General Public
+# License as published by the Free Software Foundation; either
+# version 2 of the License, or (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Library General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU Library General Public
+# License along with this library; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 ###Sketch Config
 #type = Import
@@ -46,7 +46,7 @@ from Sketch.warn import INTERNAL, USER, warn_tb
 
 from Sketch.load import GenericLoader, EmptyCompositeError
 
-from Sketch.Graphics import text
+from Sketch.Graphics import text, properties
 
 try:
     from xml.sax import saxlib, saxexts
@@ -98,22 +98,155 @@ def csscolor(str):
     return color
 
 
-namedcolors = {"black": csscolor("#000000"),
-               "silver": csscolor("#c0c0c0"),
-               "gray": csscolor("#808080"),
-               "white": csscolor("#FFFFFF"),
-               "maroon": csscolor("#800000"),
-               "red": csscolor("#FF0000"),
-               "purple": csscolor("#800080"),
-               "fuchsia": csscolor("#FF00FF"),
-               "green": csscolor("#008000"),
-               "lime": csscolor("#00FF00"),
-               "olive": csscolor("#808000"),
-               "yellow": csscolor("#FFFF00"),
-               "navy": csscolor("#000080"),
-               "blue": csscolor("#0000FF"),
-               "teal": csscolor("#008080"),
-               "aqua": csscolor("#00FFFF")}
+namedcolors = {
+    'aliceblue': csscolor('#f0f8ff'),
+    'antiquewhite': csscolor('#faebd7'),
+    'aqua': csscolor('#00ffff'),
+    'aquamarine': csscolor('#7fffd4'),
+    'azure': csscolor('#f0ffff'),
+    'beige': csscolor('#f5f5dc'),
+    'bisque': csscolor('#ffe4c4'),
+    'black': csscolor('#000000'),
+    'blanchedalmond': csscolor('#ffebcd'),
+    'blue': csscolor('#0000ff'),
+    'blueviolet': csscolor('#8a2be2'),
+    'brown': csscolor('#a52a2a'),
+    'burlywood': csscolor('#deb887'),
+    'cadetblue': csscolor('#5f9ea0'),
+    'chartreuse': csscolor('#7fff00'),
+    'chocolate': csscolor('#d2691e'),
+    'coral': csscolor('#ff7f50'),
+    'cornflowerblue': csscolor('#6495ed'),
+    'cornsilk': csscolor('#fff8dc'),
+    'crimson': csscolor('#dc143c'),
+    'cyan': csscolor('#00ffff'),
+    'darkblue': csscolor('#00008b'),
+    'darkcyan': csscolor('#008b8b'),
+    'darkgoldenrod': csscolor('#b8860b'),
+    'darkgray': csscolor('#a9a9a9'),
+    'darkgreen': csscolor('#006400'),
+    'darkgrey': csscolor('#a9a9a9'),
+    'darkkhaki': csscolor('#bdb76b'),
+    'darkmagenta': csscolor('#8b008b'),
+    'darkolivegreen': csscolor('#556b2f'),
+    'darkorange': csscolor('#ff8c00'),
+    'darkorchid': csscolor('#9932cc'),
+    'darkred': csscolor('#8b0000'),
+    'darksalmon': csscolor('#e9967a'),
+    'darkseagreen': csscolor('#8fbc8f'),
+    'darkslateblue': csscolor('#483d8b'),
+    'darkslategray': csscolor('#2f4f4f'),
+    'darkslategrey': csscolor('#2f4f4f'),
+    'darkturquoise': csscolor('#00ced1'),
+    'darkviolet': csscolor('#9400d3'),
+    'deeppink': csscolor('#ff1493'),
+    'deepskyblue': csscolor('#00bfff'),
+    'dimgray': csscolor('#696969'),
+    'dimgrey': csscolor('#696969'),
+    'dodgerblue': csscolor('#1e90ff'),
+    'firebrick': csscolor('#b22222'),
+    'floralwhite': csscolor('#fffaf0'),
+    'forestgreen': csscolor('#228b22'),
+    'fuchsia': csscolor('#ff00ff'),
+    'gainsboro': csscolor('#dcdcdc'),
+    'ghostwhite': csscolor('#f8f8ff'),
+    'gold': csscolor('#ffd700'),
+    'goldenrod': csscolor('#daa520'),
+    'gray': csscolor('#808080'),
+    'green': csscolor('#008000'),
+    'greenyellow': csscolor('#adff2f'),
+    'grey': csscolor('#808080'),
+    'honeydew': csscolor('#f0fff0'),
+    'hotpink': csscolor('#ff69b4'),
+    'indianred': csscolor('#cd5c5c'),
+    'indigo': csscolor('#4b0082'),
+    'ivory': csscolor('#fffff0'),
+    'khaki': csscolor('#f0e68c'),
+    'lavender': csscolor('#e6e6fa'),
+    'lavenderblush': csscolor('#fff0f5'),
+    'lawngreen': csscolor('#7cfc00'),
+    'lemonchiffon': csscolor('#fffacd'),
+    'lightblue': csscolor('#add8e6'),
+    'lightcoral': csscolor('#f08080'),
+    'lightcyan': csscolor('#e0ffff'),
+    'lightgoldenrodyellow': csscolor('#fafad2'),
+    'lightgray': csscolor('#d3d3d3'),
+    'lightgreen': csscolor('#90ee90'),
+    'lightgrey': csscolor('#d3d3d3'),
+    'lightpink': csscolor('#ffb6c1'),
+    'lightsalmon': csscolor('#ffa07a'),
+    'lightseagreen': csscolor('#20b2aa'),
+    'lightskyblue': csscolor('#87cefa'),
+    'lightslategray': csscolor('#778899'),
+    'lightslategrey': csscolor('#778899'),
+    'lightsteelblue': csscolor('#b0c4de'),
+    'lightyellow': csscolor('#ffffe0'),
+    'lime': csscolor('#00ff00'),
+    'limegreen': csscolor('#32cd32'),
+    'linen': csscolor('#faf0e6'),
+    'magenta': csscolor('#ff00ff'),
+    'maroon': csscolor('#800000'),
+    'mediumaquamarine': csscolor('#66cdaa'),
+    'mediumblue': csscolor('#0000cd'),
+    'mediumorchid': csscolor('#ba55d3'),
+    'mediumpurple': csscolor('#9370db'),
+    'mediumseagreen': csscolor('#3cb371'),
+    'mediumslateblue': csscolor('#7b68ee'),
+    'mediumspringgreen': csscolor('#00fa9a'),
+    'mediumturquoise': csscolor('#48d1cc'),
+    'mediumvioletred': csscolor('#c71585'),
+    'midnightblue': csscolor('#191970'),
+    'mintcream': csscolor('#f5fffa'),
+    'mistyrose': csscolor('#ffe4e1'),
+    'moccasin': csscolor('#ffe4b5'),
+    'navajowhite': csscolor('#ffdead'),
+    'navy': csscolor('#000080'),
+    'oldlace': csscolor('#fdf5e6'),
+    'olive': csscolor('#808000'),
+    'olivedrab': csscolor('#6b8e23'),
+    'orange': csscolor('#ffa500'),
+    'orangered': csscolor('#ff4500'),
+    'orchid': csscolor('#da70d6'),
+    'palegoldenrod': csscolor('#eee8aa'),
+    'palegreen': csscolor('#98fb98'),
+    'paleturquoise': csscolor('#afeeee'),
+    'palevioletred': csscolor('#db7093'),
+    'papayawhip': csscolor('#ffefd5'),
+    'peachpuff': csscolor('#ffdab9'),
+    'peru': csscolor('#cd853f'),
+    'pink': csscolor('#ffc0cb'),
+    'plum': csscolor('#dda0dd'),
+    'powderblue': csscolor('#b0e0e6'),
+    'purple': csscolor('#800080'),
+    'red': csscolor('#ff0000'),
+    'rosybrown': csscolor('#bc8f8f'),
+    'royalblue': csscolor('#4169e1'),
+    'saddlebrown': csscolor('#8b4513'),
+    'salmon': csscolor('#fa8072'),
+    'sandybrown': csscolor('#f4a460'),
+    'seagreen': csscolor('#2e8b57'),
+    'seashell': csscolor('#fff5ee'),
+    'sienna': csscolor('#a0522d'),
+    'silver': csscolor('#c0c0c0'),
+    'skyblue': csscolor('#87ceeb'),
+    'slateblue': csscolor('#6a5acd'),
+    'slategray': csscolor('#708090'),
+    'slategrey': csscolor('#708090'),
+    'snow': csscolor('#fffafa'),
+    'springgreen': csscolor('#00ff7f'),
+    'steelblue': csscolor('#4682b4'),
+    'tan': csscolor('#d2b48c'),
+    'teal': csscolor('#008080'),
+    'thistle': csscolor('#d8bfd8'),
+    'tomato': csscolor('#ff6347'),
+    'turquoise': csscolor('#40e0d0'),
+    'violet': csscolor('#ee82ee'),
+    'wheat': csscolor('#f5deb3'),
+    'white': csscolor('#ffffff'),
+    'whitesmoke': csscolor('#f5f5f5'),
+    'yellow': csscolor('#ffff00'),
+    'yellowgreen': csscolor('#9acd32'),
+}
 
 join = {'miter': const.JoinMiter,
         'round': const.JoinRound,
@@ -246,6 +379,14 @@ class SVGHandler(saxlib.HandlerBase):
         self._print('initsvg', width, height)
         self.trafo = Trafo(1, 0, 0, -1, 0, height)
         self.basetrafo = self.trafo
+        # evaluate viewBox
+        # FIXME: Handle preserveAspectRatio as well
+        viewbox = attrs.get("viewBox", "")
+        if viewbox:
+            vx, vy, vwidth, vheight = map(float, split(viewbox))
+            t = Scale(width / vwidth, height / vheight)
+            t = t(Translation(-vx, -vy))
+            self.trafo = self.trafo(t)
         self._print("basetrafo", self.basetrafo)
 
     def parse_style(self, style):
@@ -278,6 +419,13 @@ class SVGHandler(saxlib.HandlerBase):
                 self.style.line_pattern = SolidPattern(color)
         elif key == 'stroke-width':
             width = self.user_length(val)
+            # Multiply the width with a value taken from the
+            # transformation matrix because so far transforming an
+            # object in Sketch does not affect the stroke width in any
+            # way. Thus we have to do that explicitly here.
+            # FIXME: using m11 is not really the best approach but in
+            # many cases better than using the width as is.
+            width = self.trafo.m11 * width
             self._print('width', width)
             self.style.line_width = width
         elif key == 'stroke-linejoin':
@@ -285,7 +433,14 @@ class SVGHandler(saxlib.HandlerBase):
         elif key == 'stroke-linecap':
             self.style.line_cap = cap[val]
         elif key == 'font-family':
-            self.style.font = GetFont(val)
+            try:
+                # convert val to 8bit string.
+                self.style.font = GetFont(str(val))
+            except UnicodeError:
+                # If it's not ASCII we probably won't have the font, so
+                # use the default one.
+                # FIXME: Give a warning
+                pass
         elif key == 'font-size':
             self.style.font_size = self.user_length(val)
             ####self.style.font_size = float(val)
@@ -297,6 +452,17 @@ class SVGHandler(saxlib.HandlerBase):
             elif val == 'end':
               self.halign = text.ALIGN_RIGHT
 
+    def set_loader_style(self, allow_font = 0):
+        # Copy self.style to loader.
+        # If allow_font is false (the default) do not copy the font
+        # properties.
+        property_types = properties.property_types
+        style = self.style.Copy()
+        if not allow_font:
+            for name in style.__dict__.keys():
+                if property_types.get(name)==properties.FontProperty:
+                    delattr(style, name)
+        self.loader.style = style
 
     def push_state(self):
         self.state_stack = self.style, self.trafo, self.state_stack
@@ -396,7 +562,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         apply(self.loader.ellipse, t.coeff())
             
 
@@ -421,7 +587,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         apply(self.loader.ellipse, t.coeff())
 
     def rect(self, attrs):
@@ -447,7 +613,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         apply(self.loader.rectangle, t.coeff())
 
     def polyline(self, attrs):
@@ -466,7 +632,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         self.loader.bezier(paths = (path,))
 
     def polygon(self, attrs):
@@ -487,7 +653,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         self.loader.bezier(paths = (path,))
 
     def parse_path(self, str):
@@ -685,7 +851,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         
     def end_path(self):
         if self.in_defs:
@@ -720,7 +886,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style()
         t = self.trafo(Trafo(scalex, 0, 0, scaley, x, y + height))
         self._print('image', t)
         self.loader.image(image, t)
@@ -752,7 +918,7 @@ class SVGHandler(saxlib.HandlerBase):
         style = attrs.get('style', '')
         if style:
             self.parse_style(style)
-        self.loader.style = self.style
+        self.set_loader_style(allow_font = 1)
         self.current_text=''
 
     def end_text(self):
