@@ -441,7 +441,10 @@ class ProgramDefaults(connector.Publisher):
     #   is really just a list of module names that are passed to
     #   __import__, but don't count on it.
     standard_scripts = ["Script.export_raster", "Script.simple_separation",
-                        "Script.spread", "Script.reload_image"]
+                        "Script.spread", "Script.reload_image",
+                        "Script.create_star", "Script.create_star_outline",
+                        "Script.create_spiral",
+                        ]
 
 
 preferences = ProgramDefaults()
