@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -102,7 +102,8 @@ class LoaderWithComposites:
 	    if self.composite_items or composite.can_be_empty:
 		append = composite.load_AppendObject
 		for item in self.composite_items:
-		    append(item)
+                    if self.check_object(item):
+                        append(item)
 		composite.load_Done()
 		self.__pop()
 		self.append_object(composite)
@@ -116,10 +117,28 @@ class LoaderWithComposites:
 	while self.composite_stack:
 	    self.end_composite()
 
+    def check_object(self, object):
+        # Return true if object is OK. Currently this just checks
+        # whether a bezier object has at least one path and all paths
+        # have at least length 1.
+        #
+        # Furthermore, for bezier objects, guess the continuity if
+        # self.guess_continuity is true.
+        result = 1
+        if object.is_Bezier:
+            paths = object.Paths()
+            if len(paths) >= 1:
+                for path in paths:
+                    if path.len == 0:
+                        result = 0
+                        break
+            else:
+                result = 0
+            if result and self.guess_continuity:
+                object.guess_continuity()
+        return result
+
     def append_object(self, object):
-	if self.object is not None and self.guess_continuity \
-	   and self.object.is_Bezier:
-	    self.object.guess_continuity()
 	self.composite_items.append(object)
 	self.object = object
 
