@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1998, 1999, 2000, 2001, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -27,7 +27,16 @@ import config
 from Sketch import _, dgettext, gettext, bindtextdomain, message_dir, \
      Subscribe, config, const
 
+
+# All plugins are loaded as modules in the _plugin_package_name package
+# to keep the namespaces clean
+_plugin_package_name = "Sketch.Plugins"
+
+
 def create_packages(package):
+    """
+    Return the package's module and create it and it's parents if necessary
+    """
     names = split(package, '.')
     for i in range(1, len(names) + 1):
 	name = join(names[:i], '.')
@@ -35,23 +44,28 @@ def create_packages(package):
 	    module = imp.new_module(name)
 	    module.__path__ = []
 	    sys.modules[name] = module
+    return sys.modules[name]
 
 class ConfigInfo:
 
     module = None
-    package = 'Sketch.Plugins'
+    package = _plugin_package_name
 
     def __init__(self, module_name, dir, version = '1.0.0', unload = None,
-                 standard_messages = 0):
+                 load_immediately = 0, standard_messages = 0):
 	self.module_name = module_name
 	self.dir = dir
 	self.unload = unload
+        self.load_immediately = load_immediately
 	self.version = version
 	self.plugin_list.append(self)
         self.standard_messages = standard_messages
         if not self.standard_messages:
             bindtextdomain(self.module_name, message_dir)
 
+        if self.load_immediately:
+            self.load_module()
+
     def load_module(self):
 	if self.module is not None:
 	    return self.module
@@ -122,9 +136,10 @@ class ImportInfo(ConfigInfo):
 
     def __init__(self, module_name, dir, rx_magic, class_name, format_name,
 		 tk_file_type = (), version = '1.0.0', unload = None,
-                 standard_messages = 0):
+                 load_immediately = 0, standard_messages = 0):
 	ConfigInfo.__init__(self, module_name, dir, version = version,
 			    unload = unload,
+                            load_immediately = load_immediately,
                             standard_messages = standard_messages)
 	self.rx_magic = re.compile(rx_magic)
 	self.class_name = class_name
@@ -161,9 +176,10 @@ class ExportInfo(ConfigInfo):
 
     def __init__(self, module_name, dir, format_name, tk_file_type = (),
 		 extensions = (), version = '1.0.0', unload = None,
-                 standard_messages = 0):
+                 load_immediately = 0, standard_messages = 0):
 	ConfigInfo.__init__(self, module_name, dir, version = version,
 			    unload = unload,
+                            load_immediately = load_immediately,
                             standard_messages = standard_messages)
 	self.format_name = format_name
 	self.tk_file_type = tk_file_type
@@ -220,8 +236,10 @@ class PluginCompoundInfo(ConfigInfo):
 
     def __init__(self, module_name, dir, class_name, menu_text, factory = '',
                  version = '1.0.0', parameters = (), uses_selection = 0,
-                 custom_dialog = '', standard_messages = 0):
+                 custom_dialog = '', load_immediately = 0,
+                 standard_messages = 0):
 	ConfigInfo.__init__(self, module_name, dir, version = version,
+                            load_immediately = load_immediately,
                             standard_messages = standard_messages)
 	self.class_name = class_name
         self.factory = factory
@@ -313,8 +331,19 @@ def _search_dir(dir, recurse, package = 'Sketch.Plugins'):
 	return
     for file in files:
 	filename = os.path.join(dir, file)
-	if recurse and os.path.isdir(filename):
-	    _search_dir(filename, recurse - 1, package + '.' + file)
+        if os.path.isdir(filename):
+            if file == "Lib":
+                # A Lib directory on the plugin path. It's assumed to
+                # hold library files for some plugin. Append it to the
+                # current package's .Lib package's __path__ so that it's
+                # modules can be imported by prefixing their names with
+                # Lib.
+                lib_pkg = create_packages(package + '.Lib')
+                lib_pkg.__path__.append(filename)
+            elif recurse:
+                # an ordinary directory and we should recurse into it to
+                # find more modules, so do that.
+                _search_dir(filename, recurse - 1, package + '.' + file)
 	elif filename[-3:] == '.py':
 	    try:
 		module_name = os.path.splitext(os.path.basename(filename))[0]
