@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -1302,15 +1302,15 @@ def insert_node_at(path, at):
             newpath.SetContinuity(-1, ContSmooth)
         p1, p2 = control
         p1, p2, q, p3, p4 = subdivide(newpath.Node(-1), p1, p2, node, t)
-        newpath.AppendBezier(p1, p2, q, ContSymmetrical)
+        newpath.AppendBezier(p1, p2, q, ContSmooth)
         newpath.select_segment(-1)
         if cont == ContSymmetrical:
             cont = ContSmooth
         newpath.AppendBezier(p3, p4, node, cont)
     copy_path(newpath, path, index + 2)
     if path.closed:
-	newpath.ClosePath()
-	newpath.SetContinuity(-1, path.Continuity(-1))
+        newpath.ClosePath()
+        newpath.SetContinuity(-1, path.Continuity(-1))
     return newpath
 
 def split_path_at(path, at):
