@@ -2214,31 +2214,35 @@ class EditDocument(SketchDocument, QueueingPublisher):
 	return (self.update_style_dependencies, style)
 
     def UpdateDynamicStyleSel(self):
-	if len(self.selection) == 1:
-	    self.begin_transaction(_("Update Style"), clear_selection_rect = 0)
-	    try:
-		try:
-		    properties = self.CurrentProperties()
-		    # XXX hack
-		    for style in properties.stack:
-			if style.is_dynamic:
-			    break
-		    else:
-			return
-		    undo = []
-		    for attr in dir(style):
-			if attr not in ('name', 'is_dynamic'):
-			    undo.append(style.SetProperty(attr,
-							  getattr(properties,
-								  attr)))
-		    undo.append(properties.AddStyle(style))
-		    undo = (UndoAfter, CreateListUndo(undo),
-			    self.update_style_dependencies(style))
-		    self.add_undo(undo)
-		except:
-		    self.abort_transaction()
-	    finally:
-		self.end_transaction()
+        if len(self.selection) == 1:
+            self.begin_transaction(_("Update Style"), clear_selection_rect = 0)
+            try:
+                try:
+                    properties = self.CurrentProperties()
+                    # XXX hack
+                    for style in properties.stack:
+                        if style.is_dynamic:
+                            break
+                    else:
+                        return
+                    undo = []
+                    # we used to use dir(style) to get at the list of
+                    # instance variables of style. In Python 2.2 dir
+                    # returns class attributes as well. So we use
+                    # __dict__.keys() now.
+                    for attr in style.__dict__.keys():
+                        if attr not in ('name', 'is_dynamic'):
+                            undo.append(style.SetProperty(attr,
+                                                          getattr(properties,
+                                                                  attr)))
+                    undo.append(properties.AddStyle(style))
+                    undo = (UndoAfter, CreateListUndo(undo),
+                            self.update_style_dependencies(style))
+                    self.add_undo(undo)
+                except:
+                    self.abort_transaction()
+            finally:
+                self.end_transaction()
 
     def CanCreateStyle(self):
 	if len(self.selection) == 1:
