@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -1118,7 +1118,8 @@ class GraphicsDevice(SimpleGC, CommonDevice):
 	    self.DrawRectangle(Point(0, 0), Point(w, h))
 	    self.PopTrafo()
 	else:
-	    self.DrawImage(data.image, trafo)
+	    resolution = config.preferences.eps_preview_resolution
+	    self.DrawImage(data.image, trafo(Scale(72.0 / resolution)))
 
     #
     #
