@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -544,6 +544,9 @@ class HatchingPattern(Pattern):
     def Spacing(self):
 	return self.spacing
 
+    def Width(self):
+        return self.width
+
     def Transform(self, trafo, rects = None):
 	# XXX: should spacing be transformed as well? Should the pattern be
 	# transformed at all?
