@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -69,13 +69,7 @@ class PostScriptDevice(CommonDevice):
                  embed_fonts = 0, **options):
 	if as_eps and not bounding_box:
 	    raise ValueError, 'bounding_box required for EPS'
-        
-	if type(file) == StringType:
-	    file = open(file, 'w')
-	    self.close_file = 1
-	else:
-	    self.close_file = 0
-	self.file = file
+
 	self.fill = 0; self.line = 0
 	self.properties = None
 	self.line = defaultLineStyle
@@ -94,6 +88,35 @@ class PostScriptDevice(CommonDevice):
 				   printable = self.draw_printable)
 	    self.needed_resources.update(self.include_resources)
 
+        # take care of the case where we're writing to an EPS that's
+        # referenced by the document. This is a bit tricky. If the file
+        # argument is an already opened file object, all hope is lost.
+        # If it's a filename we look at all EPS files contained in the
+        # document and load them into memory if and only if they are the
+        # file we're writing into.
+
+        # dictionary used as a set of ids of EpsData objects.
+        self.loaded_eps_files = {}
+
+        # the original contents of the file we're writing to if it is
+        # actually referenced.
+        self.loaded_eps_contents = None
+
+        # search through the document if file is a string with the name
+        # of an existing file.
+        if isinstance(file, StringType) and os.path.exists(file):
+            self.filename = file
+            document.WalkHierarchy(self.handle_writes_to_embedded_eps,
+                                   visible = self.draw_visible,
+                                   printable = self.draw_printable)
+
+        if type(file) == StringType:
+	    file = open(file, 'w')
+	    self.close_file = 1
+	else:
+	    self.close_file = 0
+	self.file = file
+
 	if rotate:
 	    width, height = document.PageSize()
 	    llx, lly, urx, ury = bounding_box
@@ -198,6 +221,18 @@ class PostScriptDevice(CommonDevice):
 	if obj.is_Eps:
 	    self.needed_resources.update(obj.PSNeededResources())
 
+    def handle_writes_to_embedded_eps(self, obj):
+        if obj.is_Eps:
+            data = obj.Data()
+            # we only need to investigate this if we've not processed it yet
+            if not self.loaded_eps_files.has_key(id(data)):
+                filename = data.Filename()
+                if os.path.samefile(self.filename, filename):
+                    # it's the same file we're going to write into, so load it
+                    self.loaded_eps_files[id(data)] = 1
+                    if self.loaded_eps_contents is None:
+                        self.loaded_eps_contents = open(filename).read()
+
     trailer_written = 0
     def Close(self):
 	if self.file is not None and not self.file.closed:
@@ -312,7 +347,7 @@ class PostScriptDevice(CommonDevice):
 	self.file.write('%g %g m %g %g l s\n' % (tuple(start) + tuple(end)))
 
     def DrawLineXY(self, x1, y1, x2, y2):
-	self.file.write('%g %g m %g %g l s\n' % (x1, y1, x2, y1))
+	self.file.write('%g %g m %g %g l s\n' % (x1, y1, x2, y2))
 
     def DrawRectangle(self, start, end):
 	self.file.write('%g %g %g %g R s\n' % (tuple(start) + tuple(end)))
@@ -461,7 +496,7 @@ class PostScriptDevice(CommonDevice):
     def set_font(self, font, size):
 	spec = (font.PostScriptName(), size)
 	if self.current_font != spec:
-	    self.file.write('/%s %d sf\n' % spec)
+	    self.file.write('/%s %g sf\n' % spec)
 	    self.current_font = spec
 
     def DrawText(self, text, trafo, clip = 0, cache = None):
@@ -473,7 +508,7 @@ class PostScriptDevice(CommonDevice):
 	    self.set_font(font, self.properties.font_size)
 	    write('(%s)\n' % quote_ps_string(text))
 	    if trafo.matrix() == IdentityMatrix:
-		write('%d %d ' % tuple(trafo.offset()))
+		write('%g %g ' % tuple(trafo.offset()))
 	    else:
 		write('[%g %g %g %g %g %g] ' % trafo.coeff())
 	    if self.proc_fill:
@@ -506,7 +541,7 @@ class PostScriptDevice(CommonDevice):
 	self.set_font(font, font_size)
 	write('(%s) ' % quote_ps_string(text))
 	if trafo.matrix() == IdentityMatrix:
-	    write('%d %d ' % tuple(trafo.offset()))
+	    write('%g %g ' % tuple(trafo.offset()))
 	else:
 	    write('[%g %g %g %g %g %g] ' % trafo.coeff())
 	if self.proc_fill:
@@ -557,14 +592,17 @@ class PostScriptDevice(CommonDevice):
 
 
     def DrawEps(self, data, trafo):
-	write = self.file.write
-	write('%g %g %g %g ' % (data.Start() + data.Size()))
-	write('[%g %g %g %g %g %g]\n' % trafo.coeff())
-	write('skeps\n')
-	write('%%%%BeginDocument: %s\n' % data.Filename())
-	data.WriteLines(self.file)
-	write('\n%%EndDocument\n')
-	write('skepsend\n')
+        write = self.file.write
+        write('%g %g %g %g ' % (data.Start() + data.Size()))
+        write('[%g %g %g %g %g %g]\n' % trafo.coeff())
+        write('skeps\n')
+        write('%%%%BeginDocument: %s\n' % data.Filename())
+        if self.loaded_eps_files.has_key(id(data)):
+            write(self.loaded_eps_contents)
+        else:
+            data.WriteLines(self.file)
+        write('\n%%EndDocument\n')
+        write('skepsend\n')
 
     def DrawGuideLine(self, *args):
 	pass
@@ -612,28 +650,35 @@ class PostScriptDevice(CommonDevice):
 
     def TileImage(self, image, trafo):
 	width, height = image.size
-	length = width * height * len(image.mode)
+        if image.mode == 'RGBA':
+            # We don't support transparency in textures, so we simply
+            # treat RGBA as as RGB images
+            mode = 'RGB'
+        else:
+            mode = image.mode
+	length = width * height * len(mode)
 	if length > 65536:
 	    # the tile image is too large to fit into a PostScript
 	    # string. Resize it.
 	    #warn(USER, "Image data to big for tiling, resizing...")
 	    ratio = float(width) / height
-	    max_size = 65536 / len(image.mode)
+	    max_size = 65536 / len(mode)
 	    width = int(sqrt(max_size * ratio))
 	    height = int(sqrt(max_size / ratio))
 	    tile = image.im.resize((width, height))
-	    length = width * height * len(image.mode)
+	    length = width * height * len(mode)
 	    trafo = trafo(Scale(float(image.size[0]) / width,
 				float(image.size[1]) / height))
 	else:
 	    tile = image.im
 	write = self.file.write
 	write('%d %d %d [%g %g %g %g %g %g]\n'
-	      % ((width, height, len(image.mode)) + trafo.coeff()))
+	      % ((width, height, len(mode)) + trafo.coeff()))
 	digits = length * 2	# 2 digits per byte
 	lines = (digits - 1) / 80 + 1
 	write('%%%%BeginData: %d Hex Lines\n' % (lines + 1))
 	write("tileimage\n")
+        # write_ps_hex treats RGBA like RGB
 	_sketch.write_ps_hex(tile, self.file)
 	write('%%EndData\n')
 
