@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2002 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2002, 2003 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 #
 
 
-import os, string, re
+import os, string, re, stat
 
 # Return the value of the environment variable S if present, None
 # otherwise. In Python 1.5 one might use os.environ.get(S) instead...
@@ -152,7 +152,6 @@ def create_directory(dir):
     create_directory(parent)
     os.mkdir(dir, 0777)
 
-
 # Make a backup of FILENAME if it exists by renaming it to its
 # backupname (a ~ appended)
 
@@ -214,11 +213,7 @@ def flatten(list):
 #       String Manipulation
 #
 
-rx_format = re.compile(r'%((?P<percent>%)'
-                       r'|(\((?P<item>[a-zA-Z_]+)\)'
-                       r'\[(?P<converter>[a-zA-Z_]+)\]))')
-
-rx_format = re.compile(r'%\((?P<item>[a-zA-Z_]+)\)'
+rx_format = re.compile(r'%\((?P<item>[a-zA-Z_0-9]+)\)'
                        r'\[(?P<converter>[a-zA-Z_]+)\]')
 
 def format(template, converters, dict):
