@@ -1,5 +1,5 @@
 /* Sketch - A Python-based interactive drawing program
- * Copyright (C) 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
+ * Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 by Bernhard Herzog
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -117,7 +117,7 @@ SKCurve_New(int length)
     self->segments = malloc(length * sizeof(CurveSegment));
     if (!self->segments)
     {
-	free(self);
+	PyMem_DEL(self);
 	return PyErr_NoMemory();
     }
     self->allocated = length;
@@ -218,7 +218,7 @@ static void
 curve_dealloc(SKCurveObject * self)
 {
     free(self->segments);
-    free(self);
+    PyMem_DEL(self);
     paths_allocated--;
 }
 
