@@ -1,5 +1,5 @@
 /* Sketch - A Python-based interactive drawing program
- * Copyright (C) 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
+ * Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 by Bernhard Herzog
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -103,7 +103,7 @@ skcolor_dealloc(SKColorObject * self)
     self->ob_type = (PyTypeObject*)free_list;
     free_list = self;
 #else
-    free(self);
+    PyMem_DEL(self);
 #endif
 #if SKCOLOR_COUNT_ALLOC
     skcolor_allocated--;
@@ -594,7 +594,7 @@ SKVisual_FromXVisualInfo(Display *display, XVisualInfo *info, PyObject * args)
     self->visualinfo = PyMem_NEW(XVisualInfo, 1);
     if (!self->visualinfo)
     {
-	free(self);
+	PyMem_DEL(self);
 	return PyErr_NoMemory();
     }
     memcpy(self->visualinfo, info, sizeof(XVisualInfo));
@@ -629,7 +629,7 @@ skvisual_dealloc(SKVisualObject * self)
     if (self->free_extra)
 	self->free_extra(self);
     free(self->visualinfo);
-    free(self);
+    PyMem_DEL(self);
 }
 
 static PyObject *
