@@ -1,5 +1,5 @@
 /* Sketch - A Python-based interactive drawing program
- * Copyright (C) 1996, 1997, 1998, 2001 by Bernhard Herzog
+ * Copyright (C) 1996, 1997, 1998, 2001, 2002 by Bernhard Herzog
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -62,7 +62,7 @@ SKPoint_FromXY(SKCoord x, SKCoord y)
 static void
 skpoint_dealloc(SKPointObject * self)
 {
-    free(self);
+    PyMem_DEL(self);
 #if SKPOINT_COUNT_ALLOC
     allocated -= 1;
 #endif
