@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -545,7 +545,14 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	try:
 	    state = event.state
 	    sym = event.keysym
-	    char = event.char
+            char = event.char
+            try:
+                char = self.tk.utf8_to_latin1(char)
+            except AttributeError:
+                # we're using Python's own _tkinter. FIXME: we should
+                # handle this case properly, bit it only happens with
+                # python <= 1.5.1, so it shouldn't be much of a problem
+                pass
 	    # key events should probably be also handled by the modes
 	    if sym == 'Escape':
 		self.cancel_current_mode()
