@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@ from Sketch.Lib import util
 from Sketch.warn import warn, warn_tb, INTERNAL, USER
 from Sketch import _, config, load, plugins, SketchVersion
 from Sketch import Publisher, Point, EmptyFillStyle, EmptyLineStyle, \
-     EmptyPattern, Document, GuideLine, PostScriptDevice, SketchLoadError
+     EmptyPattern, Document, GuideLine, PostScriptDevice, SketchError
 import Sketch
 from Sketch.Graphics import image, eps
 import Sketch.Scripting
@@ -84,10 +84,11 @@ class SketchMainWindow(Publisher):
     tk_basename = 'sketch'
     tk_class_name = 'Sketch'
 
-    def __init__(self, application, filename):
+    def __init__(self, application, filename, run_script = None):
 	self.application = application
 	self.root = application.root # XXX
 	self.filename = filename
+        self.run_script = run_script
 	self.canvas = None
 	self.document = None
 	self.commands = None
@@ -172,19 +173,20 @@ class SketchMainWindow(Publisher):
 	   name_cb = lambda: os.path.split(config.preferences.mru_files[2])[1])
     AddCmd('LoadMRU3', '', 'LoadFromFile', args = 3, key_stroke = 'M-4',
 	   name_cb = lambda: os.path.split(config.preferences.mru_files[3])[1])
-    def LoadFromFile(self, filename = None):
+    def LoadFromFile(self, filename = None, directory = None):
 	app = self.application
 	if self.save_doc_if_edited(_("Open Document")) == tkext.Cancel:
 	    return
 	if type(filename) == type(0):
 	    filename = config.preferences.mru_files[filename]
 	if not filename:
-	    dir = self.document.meta.directory
-	    if not dir:
-		dir = os.getcwd()
+            if not directory:
+                directory = self.document.meta.directory
+	    if not directory:
+		directory = os.getcwd()
 	    name = ''
 	    filename = app.GetOpenFilename(filetypes = skapp.openfiletypes(),
-					   initialdir = dir,
+					   initialdir = directory,
 					   initialfile = name)
 	    if not filename:
 		return
@@ -195,7 +197,7 @@ class SketchMainWindow(Publisher):
 	    doc = load.load_drawing(filename)
 	    self.SetDocument(doc)
 	    self.add_mru_file(filename)
-	except SketchLoadError, value:
+	except SketchError, value:
 	    app.MessageBox(title = _("Open"),
 			   message = _("An error occurred:\n") + str(value))
 	    #warn_tb(USER, "An error occurred:\n", str(value))
@@ -217,6 +219,7 @@ class SketchMainWindow(Publisher):
 	filename =  self.document.meta.fullpathname
 	native_format = self.document.meta.native_format
 	compressed_file = self.document.meta.compressed_file
+	compressed = self.document.meta.compressed
 	app = self.application
 	if use_dialog or not filename or not native_format:
 	    dir = self.document.meta.directory
@@ -236,11 +239,13 @@ class SketchMainWindow(Publisher):
 	    if not fileformat:
 		fileformat = plugins.NativeFormat
 	    compressed_file = '' # guess compression from filename
+            compressed = ''
 	else:
 	    fileformat = plugins.NativeFormat
-	self.SaveToFile(filename, fileformat, compressed_file)
+	self.SaveToFile(filename, fileformat, compressed, compressed_file)
 
-    def SaveToFile(self, filename, fileformat = None, compressed_file = ''):
+    def SaveToFile(self, filename, fileformat = None, compressed = '',
+                   compressed_file = ''):
 	app = self.application
 	try:
 	    if not self.document.meta.backup_created:
@@ -269,11 +274,15 @@ class SketchMainWindow(Publisher):
 		fileformat = plugins.NativeFormat
 	    try:
 		saver = plugins.find_export_plugin(fileformat)
-		if compressed_file:
+		if compressed:
 		    # XXX there should be a plugin interface for this kind
 		    # of post-processing
-		    file = os.popen('gzip -c -9 > ' + compressed_file, 'w')
-		    saver(self.document, filename, file = file)
+                    if compressed == "gzip":
+                        cmd = 'gzip -c -9 > ' + util.sh_quote(compressed_file)
+                    elif compressed == "bzip2":
+                        cmd = 'bzip2 > ' + util.sh_quote(compressed_file)
+                    file = os.popen(cmd, 'w')
+                    saver(self.document, filename, file = file)
 		else:
 		    saver(self.document, filename)
 	    finally:
@@ -312,7 +321,14 @@ class SketchMainWindow(Publisher):
 
     AddCmd('SavePS', _("Save as PostScript..."), key_stroke = 'M-p')
     def SavePS(self, filename = None):
-	app = self.application
+        app = self.application
+        bbox = self.document.BoundingRect(visible = 0, printable = 1)
+        if bbox is None:
+            app.MessageBox(title = _("Save As PostScript"),
+                           message = _("The document doesn't have "
+                                       "any printable layers."),
+                           icon = "warning")
+            return
 	if not filename:
 	    dir = self.document.meta.ps_directory
 	    if not dir:
@@ -328,26 +344,23 @@ class SketchMainWindow(Publisher):
 					   initialfile = name)
 	    if not filename:
 		return
-	try:
-	    bbox = self.document.BoundingRect(visible = 0, printable = 1)
-	    ps_dev = PostScriptDevice(filename, as_eps = 1,
-				      bounding_box = tuple(bbox),
-				      For = util.get_real_username(),
-				      CreationDate = util.current_date(),
-				      Title = os.path.basename(filename),
-				      document = self.document)
-	    self.document.Draw(ps_dev)
-	    ps_dev.Close()
-	    self.document.meta.ps_directory = os.path.split(filename)[0]
-
-	except IOError, value:
-	    app.MessageBox(title = _("Save As PostScript"),
-			   message = _("Cannot save %(filename)s:\n"
-				       "%(message)s") \
-			   % {'filename':`os.path.split(filename)[1]`,
-			      'message':value[1]},
-			   icon = 'warning')
-	    return
+        try:
+            ps_dev = PostScriptDevice(filename, as_eps = 1,
+                                      bounding_box = tuple(bbox),
+                                      For = util.get_real_username(),
+                                      CreationDate = util.current_date(),
+                                      Title = os.path.basename(filename),
+                                      document = self.document)
+            self.document.Draw(ps_dev)
+            ps_dev.Close()
+            self.document.meta.ps_directory = os.path.split(filename)[0]
+        except IOError, value:
+            app.MessageBox(title = _("Save As PostScript"),
+                           message = _("Cannot save %(filename)s:\n"
+                                   "%(message)s") \
+                           % {'filename':`os.path.split(filename)[1]`,
+                              'message':value[1]},
+                           icon = 'warning')
 
     def add_mru_file(self, filename):
 	if filename:
@@ -385,7 +398,7 @@ class SketchMainWindow(Publisher):
 		filename = os.path.join(os.getcwd(), filename)
 	    doc = load.load_drawing(filename)
 	    group = doc.as_group()
-	except SketchLoadError, value:
+	except SketchError, value:
 	    app.MessageBox(title = _("Insert Document"),
 			   message = _("An error occurred:\n") + str(value))
 	    #warn_tb(USER, "An error occurred:\n", str(value))
@@ -497,10 +510,13 @@ class SketchMainWindow(Publisher):
 	   'CreateDialog', args = ('reloaddlg', 'ReloadPanel'))
 
     def CreatePluginDialog(self, info):
-	from plugindlg import PluginPanel
-	dialog = PluginPanel(self.root, self, self.document, info)
-	dialog.Subscribe(CLOSED, self.__dlg_closed, info.class_name)
-	self.dialogs[info.class_name] = dialog
+        if info.HasCustomDialog():
+            dialog = info.CreateCustomDialog(self.root, self, self.document)
+        else:
+            from plugindlg import PluginPanel
+            dialog = PluginPanel(self.root, self, self.document, info)
+        dialog.Subscribe(CLOSED, self.__dlg_closed, info.class_name)
+        self.dialogs[info.class_name] = dialog
 
     AddCmd('SetOptions', _("Options..."))
     def SetOptions(self):
@@ -531,8 +547,21 @@ class SketchMainWindow(Publisher):
     def Run(self):
 	#self.root.wait_visibility(self.canvas)
 	if self.filename:
-	    self.LoadFromFile(self.filename)
+            if os.path.isdir(self.filename):
+                filename = ''
+                directory = self.filename
+            else:
+                filename = self.filename
+                directory = ''
+            self.LoadFromFile(filename, directory = directory)
 	    self.filename = ''
+        if self.run_script:
+            from Sketch.Scripting.script import Context
+            dict = {'context': Context()}
+            try:
+                execfile(self.run_script, dict)
+            except:
+                warn_tb(USER, _("Error running script `%s'"), self.run_script)
 	self.application.Mainloop()
 
     AddCmd('Exit', _("Exit"), key_stroke = ('M-Q', 'M-F4'))
@@ -682,7 +711,7 @@ class SketchMainWindow(Publisher):
 	for name, plugin in items:
             if plugin.UsesSelection():
                 entries.append((plugin.menu_text, group, plugin))
-	    elif plugin.HasParameters():
+	    elif plugin.HasParameters() or plugin.HasCustomDialog():
 		entries.append((plugin.menu_text + '...', dialog, plugin))
 	    else:
 		entries.append((plugin.menu_text, place, plugin))
@@ -755,6 +784,8 @@ class SketchMainWindow(Publisher):
 		    None,
 		    cmds.ToggleOutlineMode,
 		    cmds.TogglePageOutlineMode,
+                    None,
+		    cmds.ToggleCrosshairs,
 		    None,
 		    self.commands.LoadPalette
 		    ])
@@ -975,7 +1006,7 @@ class SketchMainWindow(Publisher):
     AddCmd('AboutBox', _("About..."))
     def AboutBox(self):
 	abouttext = _("Sketch (%(version)s)\n"
-		      "(c) 1996-1999 by Bernhard Herzog\n\n"
+		      "(c) 1996-2003 by Bernhard Herzog\n\n"
 		      "Versions:\n"
 		      "Python:\t%(py)s\tTcl:\t%(tcl)s\n"
 		      "Tkinter:\t%(tkinter)s\tTk:\t%(tk)s") \
@@ -1026,8 +1057,9 @@ class SketchMainWindow(Publisher):
 
     AddCmd('python_prompt', 'Python Prompt', key_stroke = 'F11')
     def python_prompt(self):
-	import prompt
-	prompt.PythonPrompt()
+        if config.preferences.show_special_menu:
+            import prompt
+            prompt.PythonPrompt()
 
 
     #
@@ -1052,8 +1084,8 @@ class SketchMainWindow(Publisher):
 						    initialfile = initialfile)
 	return filename
 
-    AddCmd('CreateImage', _("Load Image..."), bitmap = pixmaps.Image,
-	   subscribe_to = None)
+    AddCmd('CreateImage', _("Load Raster/EPS Image..."),
+           bitmap = pixmaps.Image, subscribe_to = None)
     def CreateImage(self, filename = None):
 	if not filename:
 	    filename = self.GetOpenImageFilename(title = _("Load Image"),
