@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -133,7 +133,7 @@ class PrintPanel(SketchPanel):
 
 
     def do_print(self):
-	app = self.main_window.application
+        app = self.main_window.application
         bbox = self.document.BoundingRect(visible = 0, printable = 1)
         if bbox is None:
             app.MessageBox(title = _("Save As PostScript"),
@@ -141,40 +141,45 @@ class PrintPanel(SketchPanel):
                                        "any printable layers."),
                            icon = "warning")
             return
-	try:
-	    filename = ''
-	    file = None
-	    if self.print_dest.get() == 'file':
-		# print to file
-		filename = self.print_filename.get()
-		file = open(filename, 'w')
-		title = os.path.basename(filename)
-	    else:
-		file = os.popen(self.print_command.get(), 'w')
-		title = 'sketch'
-	    try:
-		dev = Sketch.PostScriptDevice
-		ps_dev = dev(file, as_eps = 1, bounding_box = tuple(bbox),
-			     rotate = self.var_rotate.get(),
-			     For = util.get_real_username(),
-			     CreationDate = util.current_date(), Title = title,
-			     document = self.document)
-		self.document.Draw(ps_dev)
-		ps_dev.Close()
-		if filename:
-		    self.document.meta.ps_filename = filename
-		    self.document.meta.ps_directory =os.path.split(filename)[0]
-	    finally:
-		if file is not None:
-		    file.close()
+        try:
+            filename = ''
+            file = None
+            if self.print_dest.get() == 'file':
+                # print to file
+                filename = self.print_filename.get()
+                # use filename as file just in case the user is trying
+                # to save into an EPS that is referenced by the
+                # document. The psdevice knows how to handle such cases.
+                file = filename
+                title = os.path.basename(filename)
+            else:
+                file = os.popen(self.print_command.get(), 'w')
+                title = 'sketch'
+            try:
+                dev = Sketch.PostScriptDevice
+                ps_dev = dev(file, as_eps = 1, bounding_box = tuple(bbox),
+                             rotate = self.var_rotate.get(),
+                             For = util.get_real_username(),
+                             CreationDate = util.current_date(), Title = title,
+                             document = self.document)
+                self.document.Draw(ps_dev)
+                ps_dev.Close()
+                if filename:
+                    self.document.meta.ps_filename = filename
+                    self.document.meta.ps_directory =os.path.split(filename)[0]
+            finally:
+                # close the file. Check for the close attribute first
+                # because file can be either a string or a file object.
+                if hasattr(file, "close"):
+                    file.close()
 
-	except IOError, value:
-	    app.MessageBox(title = _("Save As PostScript"),
-			   message = _("Cannot save %(filename)s:\n"
-				       "%(message)s") \
-			   % {'filename':`os.path.split(filename)[1]`,
-			      'message':value[1]},
-			   icon = 'warning')
-	    return
-	except:
-	    warn_tb(INTERNAL, 'printing to %s', file)
+        except IOError, value:
+            app.MessageBox(title = _("Save As PostScript"),
+                           message = _("Cannot save %(filename)s:\n"
+                                       "%(message)s") \
+                           % {'filename':`os.path.split(filename)[1]`,
+                              'message':value[1]},
+                           icon = 'warning')
+            return
+        except:
+            warn_tb(INTERNAL, 'printing to %s', file)
