@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -16,7 +16,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	USA
 
 import sys
-import locale
 
 from Sketch.warn import pdebug
 from Sketch import _, config, plugins, Publisher
@@ -104,15 +103,20 @@ class TkApplication:
 	self.root = Tk(screenName = screen_name,
 		       baseName = self.tk_basename,
 		       className = self.tk_class_name)
+        # Reset locale again to make sure we get properly translated
+        # messages if desired by the user. For some reason it may
+        # have been reset by Tcl/Tk.
+        # if this fails it will already have failed in
+        # Sketch/__init__.py which also prints a warning.
         try:
-            # Reset locale again to make sure we get properly translated
-            # messages if desired by the user. For some reason it may
-            # have been reset by Tcl/Tk.
-            locale.setlocale(locale.LC_MESSAGES, "")
-        except locale.Error:
-            # if this fails it will already have failed in
-            # Sketch/__init__.py which also prints a warning.
+            import locale
+        except ImportError:
             pass
+        else:
+            try:
+                locale.setlocale(locale.LC_MESSAGES, "")
+            except:
+                pass
 
 	if not geometry:
 	    # try to read geometry from resource database
