@@ -74,6 +74,10 @@ Eps Header Comments:
 			comment. Default is the current date.
   -f --for=NAME		Use NAME as the value of the `For:\' comment.
 			default is the real user name.
+
+Other Options:
+
+  -r --rotate           Rotate the drawing 90 degree counter clockwise
 '''
 
 def print_usage():
@@ -88,10 +92,11 @@ def main():
     eps_for = util.get_real_username()
     eps_date = util.current_date()
     eps_title = None
+    rotate = 0
     
     import getopt
-    opts, args = getopt.getopt(sys.argv[1:], 'hpvd:f:t:',
-			       ['help', 'noprintable', 'visible',
+    opts, args = getopt.getopt(sys.argv[1:], 'hprvd:f:t:',
+			       ['help', 'noprintable', 'rotate', 'visible',
 				'for=', 'date=', 'title='])
     
     for optchar, value in opts:
@@ -106,6 +111,8 @@ def main():
 	    eps_date = value
 	elif optchar == '-f' or optchar == '--for':
 	    eps_for = value
+        elif optchar == '-r' or optchar == '--rotate':
+            rotate = 1
 	elif optchar == '-t' or optchar == '--title':
 	    eps_title = value
     
@@ -123,7 +130,8 @@ def main():
 	eps_title = os.path.basename(filename)
 	
     sk2ps(filename, psfile, printable= draw_printable, visible = draw_visible,
-	   For = eps_for, CreationDate = eps_date, Title = eps_title)
+          For = eps_for, CreationDate = eps_date, Title = eps_title,
+          rotate = rotate)
 
 if __name__ == '__main__':
     result = main()
