@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -21,17 +21,12 @@
 # a small script to find out where Sketch is installed and launch Sketch
 #
 
-import sys, os
+import sys
 
 if sys.version < '1.5.1':
     sys.stderr.write("Sorry, Sketch requires at least Python 1.5.1\n")
     sys.exit(1)
 
-for dir in ('Lib', 'Filter', 'Pax'):
-    dir = os.path.join(sys.path[0], dir)
-    if os.path.isdir(dir):
-        sys.path.insert(1, dir)
-
 import Sketch
 
 Sketch.config.sketch_command = sys.argv[0]
